<?php

/**
 * @version    1.7.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access.
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\Utilities\ArrayHelper;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Table\Table;
use \Joomla\CMS\Date\Date;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\Filesystem\Path;
use \Joomla\Filesystem\File;
use \Joomla\Filesystem\Folder;
use \Joomla\CMS\MVC\Model\ItemModel;

/**
 * Gavoting model.
 *
 * @since  1.6
 */
class GavotingModelMotion extends ItemModel
{
    public $_item;

	/**
	 * Method to auto-populate the model state.
	 * Note. Calling getState in this method will result in recursion.
	 * @return void
	 * @since    1.6
     * @throws Exception
	 */
	protected function populateState()
	{
		$app  = Factory::getApplication('com_gavoting');
		$user = Factory::getUser();

		// Check published state
		if ((!$user->authorise('core.edit.state', 'com_gavoting')) && (!$user->authorise('core.edit', 'com_gavoting')))
		{
			$this->setState('filter.published', 1);
			$this->setState('filter.archived', 2);
		}

		// Load state from the request userState on edit or from the passed variable on default
		if (Factory::getApplication()->input->get('layout') == 'edit')
		{
			$id = Factory::getApplication()->getUserState('com_gavoting.edit.motion.id');
		}
		else
		{
			$id = Factory::getApplication()->input->get('id');
			Factory::getApplication()->setUserState('com_gavoting.edit.motion.id', $id);
		}

		$this->setState('motion.id', $id);

		// Load the parameters.
		$params       = $app->getParams();
		$params_array = $params->toArray();

		if (isset($params_array['item_id']))
		{
			$this->setState('motion.id', $params_array['item_id']);
		}

		$this->setState('params', $params);
	}

	/**
	 * Method to get an object.
	 * @param   integer $id The id of the object to get.
	 * @return  mixed    Object on success, false on failure.
     * @throws Exception
	 */
	public function getItem($id = null)
	{
            if ($this->_item === null)
            {
                $this->_item = false;

                if (empty($id))
                {
                    $id = $this->getState('motion.id');
                }

                // Get a level row instance.
                $table = $this->getTable();


                // Attempt to load the row.
                if ($table->load($id))
                {
                    
                    // Check published state.
                    if ($published = $this->getState('filter.published'))
                    {
                        if (isset($table->state) && $table->state != $published)
                        {
                            throw new Exception(Text::_('COM_GAVOTING_ITEM_NOT_LOADED'), 403);
                        }
                    }

                    // Convert the Table to a clean Object.
                    $properties  = $table->getProperties(1);
                    $this->_item = ArrayHelper::toObject($properties);

                }
            }
        
		if (isset($this->_item->created_by)) {
			$this->_item->created_by_name = Factory::getUser($this->_item->created_by)->name;
		}

		if (isset($this->_item->modified_by)) {
			$this->_item->modified_by_name = Factory::getUser($this->_item->modified_by)->name;
		}

		if (isset($this->_item->mov_id)) {
			$this->_item->mov_id_name = Factory::getUser($this->_item->mov_id)->name;
		} else {
			$this->_item->mov_id_name = '';
		}
		if (isset($this->_item->sec_id)) {
			$this->_item->sec_id_name = Factory::getUser($this->_item->sec_id)->name;
		} else {
			$this->_item->sec_id_name = 'No seconder';
		}

        return $this->_item;
    }

	/**
	 * Get an instance of JTable class
	 * @param   string $type   Name of the JTable class to get an instance of.
	 * @param   string $prefix Prefix for the table class name. Optional.
	 * @param   array  $config Array of configuration values for the JTable object. Optional.
	 * @return  JTable|bool JTable if success, false on failure.
	 */
	public function getTable($type = 'Motion', $prefix = 'GavotingTable', $config = array())
	{
		$this->addTablePath(JPATH_ADMINISTRATOR . '/components/com_gavoting/tables');

		return Table::getInstance($type, $prefix, $config);
	}

	/**
	 * Get the id of an item by alias
	 * @param   string $alias Item alias
	 * @return  mixed
	 */
	public function getItemIdByAlias($alias)
	{
            $table      = $this->getTable();
            $properties = $table->getProperties();
            $result     = null;

            if (key_exists('alias', $properties))
            {
                $table->load(array('alias' => $alias));
                $result = $table->id;
            }
            
                return $result;
            
	}

	/**
	 * Method to check in an item.
	 * @param   integer $id The id of the row to check out.
	 * @return  boolean True on success, false on failure.
	 * @since    1.6
	 */
	public function checkin($id = null)
	{
		// Get the id.
		$id = (!empty($id)) ? $id : (int) $this->getState('motion.id');
                
		if ($id)
		{
			// Initialise the table
			$table = $this->getTable();

			// Attempt to check the row in.
			if (method_exists($table, 'checkin'))
			{
				if (!$table->checkin($id))
				{
					return false;
				}
			}
		}

		return true;
                
	}

	/**
	 * Method to check out an item for editing.
	 * @param   integer $id The id of the row to check out.
	 * @return  boolean True on success, false on failure.
	 * @since    1.6
	 */
	public function checkout($id = null)
	{
		// Get the user id.
		$id = (!empty($id)) ? $id : (int) $this->getState('motion.id');
                
		if ($id)
		{
			// Initialise the table
			$table = $this->getTable();

			// Get the current user object.
			$user = Factory::getUser();

			// Attempt to check the row out.
			if (method_exists($table, 'checkout'))
			{
				if (!$table->checkout($user->get('id'), $id))
				{
					return false;
				}
			}
		}

		return true;
                
	}

	/**
	 * Publish the element
	 * @param   int $id    Item id
	 * @param   int $state Publish state
	 * @return  boolean
	 */
	public function publish($id, $state)
	{
		$table = $this->getTable();
                
		$table->load($id);
		$table->state = $state;

		return $table->store();
                
	}

	/**
	 * Method to delete an item
	 * @param   int $id Element id
	 * @return  bool
	 */
	public function delete($id)
	{
		$table = $this->getTable();

		$table->load($id);
		$table->state = -2;

		return $table->store();
                
	}

	/**
	 * Method to agree a motion
	 * @param   int $id Element id
	 * @return  bool
	 */
	public function agree($id, $user_id)
	{
		$tz = Factory::getConfig()->get('offset');
		$date = Factory::getDate('now', $tz);
		$today = date_format($date,'Y-m-d H:i:s');
		$table = $this->getTable();

		$table->load($id);
		$table->agreed = 1;
		$table->agreed_date = $today;

		return $table->store();
                
	}

    /**
     * Method to record a vote for motion
     * @param   int $pk Item primary key
     */
    public function voteFor($pk = 0)
    {
        if ($pk == 0) {
            throw new Exception(Text::_('COM_GAVOTING_ITEM_DOESNT_EXIST'), 404);
        }

        $user = Factory::getUser();
        $hasVoted = GavotingHelper::hasVoted($user->id, $pk);

        if ($hasVoted) {
            throw new Exception(Text::_('COM_GAVOTING_MOTION_ALREADY_VOTED'), 404);
        }

        if ($user->authorise('core.vote', 'com_gavoting') !== true) {
            throw new Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
        }

        $table = $this->getTable();
        $table->load($pk);
        $table->votes_for = $table->votes_for + 1;

        if ($table->store($pk) !== true) {
            throw new Exception(Text::_('JERROR_FAILED'), 501);
        }

        $voteCreated = GavotingHelper::createVote($user->id, $pk);

        return $voteCreated;
        
    }

    /**
     * Method to record a vote against motion
     * @param   int $pk Item primary key
     */
    public function voteAgainst($pk = 0)
    {
        if ($pk == 0) {
            throw new Exception(Text::_('COM_GAVOTING_ITEM_DOESNT_EXIST'), 404);
        }

        $user = Factory::getUser();
        $hasVoted = GavotingHelper::hasVoted($user->id, $pk);

        if ($hasVoted) {
            throw new Exception(Text::_('COM_GAVOTING_MOTION_ALREADY_VOTED'), 404);
        }

        if ($user->authorise('core.vote', 'com_gavoting') !== true) {
            throw new Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
        }

        $table = $this->getTable();
        $table->load($pk);
        $table->votes_against = $table->votes_against + 1;

        if ($table->store($pk) !== true) {
            throw new Exception(Text::_('JERROR_FAILED'), 501);
        }

        $voteCreated = GavotingHelper::createVote($user->id, $pk);

        return $voteCreated;
        
    }

    /**
     * Method to record a vote decision motion
     * @param   int $pk Item primary key
     */
    public function voteDecision($pk = 0)
    {
        if ($pk == 0) {
            throw new Exception(Text::_('COM_GAVOTING_ITEM_DOESNT_EXIST'), 404);
        }

        $user = Factory::getUser();
        $today = GavotingHelper::getTodaysDate();

        if ($user->authorise('core.vote', 'com_gavoting') !== true) {
            throw new Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
        }

        $table = $this->getTable();
        $table->load($pk);
        
        if ($table->votes_for != $table->votes_against) {
			$table->agreed_date = $today;
			if ($table->votes_for > $table->votes_against) {
				$table->agreed = 1;
	        } else {
				$table->agreed = 0;
			}
        } else {
            throw new Exception(Text::_('COM_GAVOTING_CAST_DECIDER'), 403);
            //Factory::getApplication()->enqueueMessage(Text::_('COM_GAVOTING_CAST_DECIDER'),'danger');
			//return $pk;
        }

        if ($table->store($pk) !== true) {
            throw new Exception(Text::_('JERROR_FAILED'), 501);
        }

        return $pk;
        
    }

}
