<?php
/**
 * @version    1.7.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

// No direct access.
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\Utilities\ArrayHelper;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Table\Table;
use \Joomla\CMS\MVC\Model\FormModel;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\Filesystem\File;
use \Joomla\Filesystem\Folder;
use \Joomla\Filesystem\Path;

/**
 * Gavoting model.
 *
 * @since  1.6
 */
class GavotingModelVoterForm extends FormModel
{
    private $item = null;

    /**
     * Method to auto-populate the model state.
     * Note. Calling getState in this method will result in recursion.
     * @return void
     * @since  1.6
     * @throws Exception
     */
    protected function populateState()
    {
        $app = Factory::getApplication('com_gavoting');

        // Load state from the request userState on edit or from the passed variable on default
        if (Factory::getApplication()->input->get('layout') == 'edit') {
                $id = Factory::getApplication()->getUserState('com_gavoting.edit.voter.id');
        } else {
                $id = Factory::getApplication()->input->get('id');
                Factory::getApplication()->setUserState('com_gavoting.edit.voter.id', $id);
        }

        $this->setState('voter.id', $id);

        // Load the parameters.
        $params       = $app->getParams();
        $params_array = $params->toArray();

        if (isset($params_array['item_id'])) {
                $this->setState('voter.id', $params_array['item_id']);
        }

        $this->setState('params', $params);
    }

    /**
     * Method to get an ojbect.
     * @param   integer $id The id of the object to get.
     * @return Object|boolean Object on success, false on failure.
     * @throws Exception
     */
    public function getItem($id = null)
    {
        if ($this->item === null)
        {
            $this->item = false;

            if (empty($id))
            {
                    $id = $this->getState('voter.id');
            }

            // Get a level row instance.
            $table = $this->getTable();

            if ($table !== false && $table->load($id))
            {
                $user = Factory::getUser();
                $id   = $table->id;
                

                $canEdit = $user->authorise('core.edit', 'com_gavoting') || $user->authorise('core.create', 'com_gavoting') || $user->authorise('core.vote', 'com_gavoting');
                
                if (!$canEdit && $user->authorise('core.edit.own', 'com_gavoting'))
                {
                        $canEdit = $user->id == $table->created_by;
                }

                if (!$canEdit)
                {
                        throw new Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
                }

                // Check published state.
                if ($published = $this->getState('filter.published'))
                {
                        if (isset($table->state) && $table->state != $published)
                        {
                                return $this->item;
                        }
                }

                // Convert the JTable to a clean JObject.
                $properties = $table->getProperties(1);
                $this->item = ArrayHelper::toObject($properties, 'JObject');
                
            }
        }
        
        // load the cat_id field with the current election year
        if (empty($this->item->cat_id)) {
			$electionYear = GavotingHelper::getElectionYear();
			$this->item->cat_id = $electionYear->id;
		}
		
		// load up the positions set
		$positions = GavotingHelper::getPositions();
		$pos_ids = array();
		foreach ($positions as $pos) {
			if ($pos->id == 0) {
				// do nothing
			} else {
				// load up the nominations set
				$pos_ids[$pos->id] = GavotingHelper::getPositionNominations($pos->id);
			}
		}

		// load up the nominations set
		$this->item->nominations = $pos_ids;

        return $this->item;
    }

    /**
     * Method to get the table
     * @param   string $type   Name of the JTable class
     * @param   string $prefix Optional prefix for the table class name
     * @param   array  $config Optional configuration array for JTable object
     * @return  JTable|boolean JTable if found, boolean false on failure
     */
    public function getTable($type = 'Voter', $prefix = 'GavotingTable', $config = array())
    {
        $this->addTablePath(JPATH_ADMINISTRATOR . '/components/com_gavoting/tables');

        return Table::getInstance($type, $prefix, $config);
    }

    /**
     * Get an item by alias
     * @param   string $alias Alias string
     * @return int Element id
     */
    public function getItemIdByAlias($alias)
    {
        $table      = $this->getTable();
        $properties = $table->getProperties();

        if (!in_array('alias', $properties))
        {
                return null;
        }

        $table->load(array('alias' => $alias));

        return $table->id;

    }

    /**
     * Method to check in an item.
     * @param   integer $id The id of the row to check out.
     * @return  boolean True on success, false on failure.
     * @since    1.6
     */
    public function checkin($id = null)
    {
        // Get the id.
        $id = (!empty($id)) ? $id : (int) $this->getState('voter.id');
        
        if ($id)
        {
            // Initialise the table
            $table = $this->getTable();

            // Attempt to check the row in.
            if (method_exists($table, 'checkin'))
            {
                if (!$table->checkin($id))
                {
                    return false;
                }
            }
        }

        return true;
        
    }

    /**
     * Method to check out an item for editing.
     * @param   integer $id The id of the row to check out.
     * @return  boolean True on success, false on failure.
     * @since    1.6
     */
    public function checkout($id = null)
    {
        // Get the user id.
        $id = (!empty($id)) ? $id : (int) $this->getState('voter.id');
        
        if ($id)
        {
            // Initialise the table
            $table = $this->getTable();

            // Get the current user object.
            $user = Factory::getUser();

            // Attempt to check the row out.
            if (method_exists($table, 'checkout'))
            {
                if (!$table->checkout($user->get('id'), $id))
                {
                    return false;
                }
            }
        }

        return true;
        
    }

    /**
     * Method to get the form.
     * The base form is loaded from XML
     * @param   array   $data     An optional array of data for the form to interogate.
     * @param   boolean $loadData True if the form is to load its own data (default case), false if not.
     * @return    JForm    A JForm object on success, false on failure
     * @since    1.6
     */
    public function getForm($data = array(), $loadData = true)
    {
 		// Get the form.
        $form = $this->loadForm('com_gavoting.voter', 'voterform', array(
                        'control'   => 'jform',
                        'load_data' => $loadData
                )
        );

        if (empty($form))
        {
                return false;
        }
        


        return $form;

    }

    /**
     * Method to get the data that should be injected in the form.
     * @return    mixed    The data for the form.
     * @since    1.6
     */
    protected function loadFormData()
    {
        $data = Factory::getApplication()->getUserState('com_gavoting.edit.voter.data', array());

        if (empty($data))
        {
            $data = $this->getItem();
        }

        return $data;
    }

    /**
     * Method to save the form data.
     * @param   array $data The form data
     * @return bool
     * @throws Exception
     * @since 1.6
     */
    public function save($data)
    {
		$data['id'] = 0;
        $data['state'] = 1;
        /* ---------------------    this is the voting user   ---------------------- */
		$mu = Factory::getUser($data['user_id']);

        $authorised = $mu->authorise('core.vote', 'com_gavoting');

        if ($authorised !== true) {
            Factory::getApplication()->enqueueMessage(Text::_('COM_GAVOTING_NOTAUTH_VOTE'), 'danger');
            return false;
        }

        /* ------------------------------------------------------------------------- */
		/* ---------   Get the parameters and other reference data ----------------- */
        /* ------------------------------------------------------------------------- */
		$params = ComponentHelper::getParams('com_gavoting');
        $proxy_type = $params->get('proxy_type'); // 0=general,1=specific,2=hybrid
        $dupl_vote = $params->get('dupl_vote');   //0=no,1=yes  only needs to be consider if proxy_type = 0

		/* -------   Check if principal/owner of vote has delegated a proxy  ------- */
		/* -------   returns an object of the proxy record or boolean on null  ----- */
		$hasProxy = GavotingHelper::checkProxySet($mu->id);

		/* -- If no proxy set for this user, count how many proxy votes this user has -- */
        $qtyProxy = !$hasProxy ? GavotingHelper::countProxyVotes($mu->id) : 0;

		if (!$proxy_type && $dupl_vote) {
			$votes = 1 + $qtyProxy;
		} else {
			$votes = 1;
		}

        /* ------------------------------------------------------------------------- */
		/* -------  Set the vote record for the Voting User as their own vote ------ */
        /* ------------------------------------------------------------------------- */
        $vote = array('id'=>0,'state'=>1,'user_id'=>$mu->id,'proxy_vote'=>0,'cat_id'=>$data['cat_id']);

        /* ------------------------------------------------------------------------- */
        /* ----------  Work through Proxy possibilies based on parameters  --------- */
        /* ----  This section will only be relevant if proxy votes are separate ---- */
        /* ------------------------------------------------------------------------- */

		if ($data['proxy_vote']) {
			$pu = Factory::getUser($data['proxy_for']);

	        /* ------------------------------------------------------------------------- */
			/* -------------------------- Check if already voted ----------------------- */
	        /* ------------------------------------------------------------------------- */
	        $hasVoted = GavotingHelper::hasVoted($pu->id);
	        if ($hasVoted) {
	            Factory::getApplication()->enqueueMessage(Text::_('COM_GAVOTING_NOMINATIONS_ALREADY_VOTED'), 'danger');
	            return false;
	        }

	        /* ------------------------------------------------------------------------- */
			/* --  Set the vote record for the Proxy vote submitted by their delegate -- */
	        /* ------------------------------------------------------------------------- */
			$proxyvote = array('id'=>0,'state'=>1,'user_id'=>$pu->id,'proxy_vote'=>1,'cat_id'=>$data['cat_id']);

			// specific and hybrid means only record vote for proxy_for
			if ($proxy_type == 1 || $proxy_type == 2) {
				// Set voteform data based on selected proxy
				self::gatherNomVotes($data['nom_ids']);
				self::recordVote($proxyvote, $pu->name, 'Proxy');
			} else {
				// general means test for dupl_vote
				if ($dupl_vote) {
					// save twice
					// record the selected nominations for the proxy
					self::gatherNomVotes($data['nom_ids']);
					self::recordVote($proxyvote, $pu->name, 'Proxy');
					// record the selected nominations for the principal
					self::gatherNomVotes($data['nom_ids']);
					self::recordVote($vote, $mu->name, 'Member');
				} else {
					// record the selected nominations for the proxy
					self::gatherNomVotes($data['nom_ids']);
					self::recordVote($proxyvote, $pu->name, 'Proxy');
				}
			}
		} else {
	        if ($qtyProxy) {

				/* ---------------  Cycle through Proxy records if necessary  -------------- */
	            $proxys = GavotingHelper::getProxyVotes($mu->id);
				
				foreach ($proxys as $proxy) {
					/* ------------------------------------------------------------------------- */
					/* -------------------------- Check if already voted ----------------------- */
			        /* ------------------------------------------------------------------------- */
			        $hasVoted = GavotingHelper::hasVoted($proxy->voter_id);
			        if ($hasVoted) {
			            Factory::getApplication()->enqueueMessage(Text::_('COM_GAVOTING_NOMINATIONS_ALREADY_VOTED'), 'danger');
			        } else {
				        /* ------------------------------------------------------------------------- */
						/* --  Set the vote record for the Proxy vote submitted by their delegate -- */
				        /* ------------------------------------------------------------------------- */
						$proxyvote = array('id'=>0,'state'=>1,'user_id'=>$proxy->voter_id,'proxy_vote'=>1,'cat_id'=>$data['cat_id']);
				        // record the selected nominations for the proxy
						self::gatherNomVotes($data['nom_ids']);
						self::recordVote($proxyvote, $proxy->name, 'Proxy');
					}
				}
		        // record the selected nominations for the principal
				self::gatherNomVotes($data['nom_ids']);
				self::recordVote($vote, $mu->name, 'Member');
	        } else {
		        // record the selected nominations for the principal
				self::gatherNomVotes($data['nom_ids']);
				self::recordVote($vote, $mu->name, 'Member');
			}
        }

        return true;
    }

    /**
     * Method to work through the nominations
     * @param   array $data
     * @return  vote/s position and nomination
     */
    public function gatherNomVotes($nom_ids)
    {
		foreach ($nom_ids as $keys => $values) {
			/* ------ strip out "nom_id" prefix to get the position id reference ---- */
            $posID = substr($keys,6);

			$position = GavotingHelper::getPosition($posID);

			if (is_array($values)) {

				if (count($values) > $position->pos_qty) {
					Factory::getApplication()->enqueueMessage(Text::sprintf('COM_GAVOTING_MULTISELECT_EXCEEDED', $position->position_name), 'danger');
					return false;
				}

                $voting = array();
				foreach ($values as $val) {
	                $voting[$posID][] = $val;
			    }
				/* ------ store the votes --------- */
				GavotingHelper::addVoteToNomination($voting);

			} else {
				$voting = array($posID => $values);
				/* ------ store the votes --------- */
				GavotingHelper::addVoteToNomination($voting);
			}
		}

        return $voting;
    }

    /**
     * Method to record vote data
     * @param   array $data
     * @return  boolean
     */
    public function recordVote($data, $name, $voteType)
    {
		$table = $this->getTable();

        if ($table->save($data) === true) {
			Factory::getApplication()->enqueueMessage($voteType . ' Vote for '.$name.' Saved', 'message');

	        return true;
        } else {
			Factory::getApplication()->enqueueMessage($voteType . ' Vote for '.$name.' Failed', 'Warning');
			return false;
        }
    }

    /**
     * Method to delete data
     * @param   int $pk Item primary key
     * @return  int  The id of the deleted item
     * @throws Exception
     * @since 1.6
     */
    public function delete($pk)
    {
        $user = Factory::getUser();

        if (empty($pk)) {
            $pk = (int) $this->getState('voter.id');
        }

        if ($pk == 0 || $this->getItem($pk) == null) {
            throw new Exception(Text::_('COM_GAVOTING_ITEM_DOESNT_EXIST'), 404);
        }

        if ($user->authorise('core.delete', 'com_gavoting') !== true) {
            throw new Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
        }

        $table = $this->getTable();
        $table->load($pk);
        $table->state = -2;

        if ($table->store($pk) !== true) {
            throw new Exception(Text::_('JERROR_FAILED'), 501);
        }

        return $pk;
        
    }

    /**
     * Method to archive data
     * @param   int $pk Item primary key
     * @return  int  The id of the deleted item
     * @throws Exception
     * @since 1.6
     */
    public function archive($pk)
    {
        $user = Factory::getUser();

        if (empty($pk)) {
            $pk = (int) $this->getState('voter.id');
        }

        if ($pk == 0 || $this->getItem($pk) == null) {
            throw new Exception(Text::_('COM_GAVOTING_ITEM_DOESNT_EXIST'), 404);
        }

        if ($user->authorise('core.delete', 'com_gavoting') !== true) {
            throw new Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
        }

        $table = $this->getTable();
        $table->load($pk);
        $table->state = 2;

        if ($table->store($pk) !== true) {
            throw new Exception(Text::_('JERROR_FAILED'), 501);
        }

        return $pk;
        
    }

    /**
     * Check if data can be saved
     * @return bool
     */
    public function getCanSave()
    {
        $table = $this->getTable();

        return $table !== false;
    }

    /**
     * Method to record the vote decisions
     */
    public function voteDecision()
    {
		return GaupdparamsHelper::rolloverElectionYear();
    }
    
}
